@@ -24,13 +24,17 @@
 
 #include <fcntl.h>
 #include <gnome.h>
+#include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <glade/glade.h>
 #include <unistd.h>
-#include "gfloppy.h"
 #include <sys/types.h>
 #include <errno.h>
+
+#include "gfloppy.h"
+#include "gfloppy-config.h"
 #include "progress.h"
+
 #define NUM_DEVICES_TO_CHECK 4
 
 extern int errno;
@@ -39,9 +43,18 @@ static GFloppy floppy;
 static GladeXML *xml;
 static GtkWidget *toplevel = NULL;
 
+gboolean prefer_mkdosfs_backend;
+
 /* Only needed if more then one device exists */
 static GList *valid_devices = NULL;
 
+
+static GFloppyFormattingMode get_selected_formatting_mode ();
+static void type_option_changed_cb     (GtkOptionMenu   *option_menu,
+					gpointer         data);
+static void set_widget_sensitive_from  (GtkToggleButton *button,
+					gpointer         data);
+
 static void
 devices_option_activated (GtkWidget *menu_item, gchar *device)
 {
@@ -62,11 +75,16 @@ start_format (void)
 {
 	pipe (floppy.message);
 
+	/* setup size field based on user request */
+	floppy.nblocks = floppy_block_size (floppy.size);
+	floppy.nbadblocks = 0;
 	floppy.pid = fork ();
+
 	if (floppy.pid < 0) {
 		g_error ("unable to fork ().\nPlease free up some resources and try again.\n");
 		_exit (1);
 	}
+
 	if (floppy.pid == 0) {
 		/* child */
 		close (floppy.message [0]);
@@ -89,10 +107,6 @@ init_commands (void)
 	const char *path;
 	char *newpath;
 
-	floppy.mke2fs_cmd = NULL;
-	floppy.mformat_cmd = NULL;
-	floppy.badblocks_cmd = NULL;
-
 	path = g_getenv ("PATH");
 
 	if (path)
@@ -103,14 +117,25 @@ init_commands (void)
 	putenv (newpath); /* Sigh, we have to leak it */
 
 	floppy.mke2fs_cmd    = g_find_program_in_path ("mke2fs");
+	floppy.mkdosfs_cmd   = g_find_program_in_path ("mkdosfs");
 	floppy.mformat_cmd   = g_find_program_in_path ("mformat");
 	floppy.badblocks_cmd = g_find_program_in_path ("mbadblocks");
 
-	if (floppy.mke2fs_cmd == NULL) {
-		g_print ("Warning:  Unable to locate mke2fs.  Please confirm it is installed and try again\n");
+	if (!floppy.mke2fs_cmd &&
+	    !floppy.mkdosfs_cmd &&
+	    !floppy.mformat_cmd) {
+		GtkWidget *error_dialog;
+
+		error_dialog = gtk_message_dialog_new (NULL, 0,
+						       GTK_MESSAGE_ERROR,
+						       GTK_BUTTONS_CLOSE,
+						       _("Neither the mke2fs nor the mkdosfs/mformat applications are installed. You can't format a floppy without one of them."));
+
+		gtk_dialog_run (GTK_DIALOG (error_dialog));
+		gtk_widget_destroy (error_dialog);
+
 		exit (1);
 	}
-
 }
 
 static void
@@ -148,6 +173,7 @@ init_devices (void)
 							 GTK_BUTTONS_CLOSE,
 							 msg);
 			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
 			exit (1);
 		}
 		/* Device is okay */
@@ -188,6 +214,7 @@ init_devices (void)
 							 GTK_BUTTONS_CLOSE,
 							 msg);
 			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
 			exit (1);
 		} else if (ok_devices_present == 1) {
 			for (i = 0; i < NUM_DEVICES_TO_CHECK; i++) {
@@ -263,24 +290,176 @@ set_floppy_extended_device (void)
 	}
 }
 
+static GFloppyFormattingMode
+get_selected_formatting_mode ()
+{
+	GtkWidget *quick_format_rb;
+	GSList *l;
+	gint i;
+
+        quick_format_rb = glade_xml_get_widget (xml, "quick_format_rb");
+
+	l = gtk_radio_button_get_group (GTK_RADIO_BUTTON (quick_format_rb));
+
+	/* Note: don't doubt of the decrementation, that's because of the
+		 group's list whose values are inserted at its head.
+		 Perhaps it may be safier to just play the "if-else" war. */
+
+	for (i = 2; i < GFLOPPY_FORMAT_END; i--, l = l->next)
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (l->data)))
+			return i;
+
+	g_assert_not_reached ();
+}
+
+static void
+type_option_changed_cb (GtkOptionMenu *option_menu,
+			gpointer       data)
+{
+	GtkOptionMenu *type_om;
+	GtkWidget *note_label;
+	GtkWidget *standard_format_rb;
+	GtkWidget *thorough_format_rb;
+	gboolean cannot_check;
+
+	type_om = GTK_OPTION_MENU (option_menu);
+	standard_format_rb = glade_xml_get_widget (xml, "standard_format_rb");
+	thorough_format_rb = glade_xml_get_widget (xml, "thorough_format_rb");
+	note_label = glade_xml_get_widget (xml, "note_label");
+
+	cannot_check = gtk_option_menu_get_history (type_om) == 1 && /* fat fs */
+		       !floppy.mkdosfs_backend &&		     /* mtools backend */
+		       floppy.badblocks_cmd == NULL;		     /* mbadblocks not installed */
+
+	if (cannot_check &&
+	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (thorough_format_rb)))
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (standard_format_rb), TRUE);
+
+	gtk_widget_set_sensitive (thorough_format_rb, !cannot_check);
+
+	if (cannot_check)
+		gtk_widget_show (note_label);
+	else
+		gtk_widget_hide (note_label);
+}
+
+static void
+set_widget_sensitive_from (GtkToggleButton *button,
+			   gpointer         data)
+{
+	GtkWidget *widget;
+	gboolean active;
+
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (data));
+
+	widget = GTK_WIDGET (data);
+	active = gtk_toggle_button_get_active (button);
+
+	gtk_widget_set_sensitive (widget, active);
+
+	if (active)
+		gtk_widget_grab_focus (widget);
+}
+
 gint
 on_toplevel_delete_event (GtkWidget *w, GdkEventKey *e)
 {
-	toplevel = NULL;
+	gtk_main_quit ();
+
 	return TRUE;
 }
 
+void
+on_close_button_clicked (GtkWidget *widget, gpointer user_data)
+{
+	gtk_main_quit ();
+}
+
+void
+on_help_button_clicked (GtkWidget *widget, gpointer user_data)
+{
+	GError *error = NULL;
+
+	gnome_help_display ("gfloppy", "intro", &error);
+
+	if (error) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+						 _("Could not display help for the "
+						   "floppy formatter.\n"
+						   "%s"),
+						 error->message);
+
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_widget_show (dialog);
+
+		g_error_free (error);
+	}
+}
+
+void
+on_format_button_clicked (GtkWidget *widget, gpointer user_data)
+{
+	GtkWidget *dialog;
+	GtkWidget *type_option;
+	GtkWidget *density_option;
+	GtkWidget *volume_name_entry;
+
+        type_option = glade_xml_get_widget (xml, "type_option");
+        density_option = glade_xml_get_widget (xml, "density_option");
+	volume_name_entry = glade_xml_get_widget (xml, "volume_name_entry");
+
+	floppy.volume_name = gtk_entry_get_text (GTK_ENTRY (volume_name_entry));
+
+	if (strstr (floppy.volume_name, " ")) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("The volume name can't contain any blank space."));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		return;
+	}
+
+	gtk_widget_set_sensitive (toplevel, FALSE);
+
+	/* If we can format in either types */
+	if (floppy.mke2fs_cmd && (floppy.mkdosfs_cmd || floppy.mformat_cmd))
+		floppy.type = gtk_option_menu_get_history (GTK_OPTION_MENU (type_option)); /* user choice */
+	else
+		floppy.type = floppy.mke2fs_cmd ? GFLOPPY_E2FS : GFLOPPY_FAT; /* otherwise just use the available one */
+
+	floppy.size = gtk_option_menu_get_history (GTK_OPTION_MENU (density_option));
+	floppy.formatting_mode = get_selected_formatting_mode ();
+
+	set_floppy_extended_device ();
+
+        start_format ();
+
+        gtk_widget_set_sensitive (toplevel, TRUE);
+}
+
 int
 main (int argc, char *argv[])
 {
-	GtkWidget *ext2_entry;
+	GConfClient *client;
+	GFloppyConfig config;
+	GtkWidget *fs_entry;
 	GtkWidget *type_option;
-	GtkWidget *icon_frame;
-	GtkWidget *icon;
-	GtkWidget *quick_format_button;
+	GtkWidget *quick_format_rb;
+	GtkWidget *default_mode_rb;
 	GtkWidget *device_combo;
+	GSList *l;
 	gchar *image;
-	gint button;
+	gint i;
 
 	struct poptOption gfloppy_opts[] = {
 		{"device", '\0', POPT_ARG_STRING, NULL, 0, NULL, NULL},
@@ -303,8 +482,6 @@ main (int argc, char *argv[])
 	/* FIXME: get the right icon */
 	image = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
                                            "document-icons/i-floppy.png", TRUE, NULL);
-
-	
 	gnome_window_icon_set_default_from_file (image);
 	g_free (image);
 
@@ -312,95 +489,102 @@ main (int argc, char *argv[])
 
 	/* Now we can set up glade */
 	glade_gnome_init();
+
         xml = glade_xml_new (GLADEDIR "/gfloppy2.glade", NULL, NULL);
 	if (xml == NULL)
 		g_error ("Cannot load/find gfloppy2.glade");
 
 	init_devices ();
-	toplevel = glade_xml_get_widget (xml, "toplevel");
 
-	quick_format_button = glade_xml_get_widget (xml, "quick_format_button");
-	icon_frame = glade_xml_get_widget (xml, "icon_frame");
-	type_option = glade_xml_get_widget (xml, "type_option");
-	toplevel = glade_xml_get_widget (xml, "toplevel");
-	device_combo = glade_xml_get_widget (xml, "device_combo");
-	glade_xml_signal_autoconnect (xml);
+	/* We get the different widgets */
 	
-	icon = gtk_image_new_from_stock (GTK_STOCK_FLOPPY, GTK_ICON_SIZE_DIALOG);
-	gtk_container_add (GTK_CONTAINER (icon_frame), icon);
-	gtk_widget_show (icon);
+	toplevel = glade_xml_get_widget (xml, "toplevel");
+	type_option = glade_xml_get_widget (xml, "type_option");
+        quick_format_rb = glade_xml_get_widget (xml, "quick_format_rb");
 
-	if (floppy.mformat_cmd == NULL) {
-		/* We don't have mtools.  Allow ext2 only. */
-		ext2_entry = glade_xml_get_widget (xml, "ext2_entry");
+	/* Signals connection */
+	
+	glade_xml_signal_autoconnect (xml);
+
+	g_signal_connect (type_option, "changed",
+			  G_CALLBACK (type_option_changed_cb), NULL);
+
+	/* GConf support */
+	client = gconf_client_get_default ();
+	gconf_client_add_dir (client, "/apps/gfloppy",
+			      GCONF_CLIENT_PRELOAD_NONE, NULL);
+
+	gfloppy_config_load (&config, client);
+
+	/* if the two FAT formatting backend are present, then the advanced
+	   setting choose for us, but we don't have to take the badblocks
+	   command in account */
+	if (floppy.mkdosfs_cmd != NULL && floppy.mformat_cmd != NULL)
+		floppy.mkdosfs_backend = config.prefer_mkdosfs_backend;
+	else
+		floppy.mkdosfs_backend = floppy.mkdosfs_cmd != NULL;
+
+	/* Sigh, sorry for using a global var for this */
+	prefer_mkdosfs_backend = config.prefer_mkdosfs_backend;
+
+	/* Sync UI from config. We don't emit any signal here, since we emit
+	   just after this sync the "changed" signal of type_option,
+	   which emit all the other necessary signals */
+	if (!strcmp (config.default_fs, GFLOPPY_CONFIG_FS_FAT))
+		gtk_option_menu_set_history (GTK_OPTION_MENU (type_option), GFLOPPY_FAT);
+	else
+		gtk_option_menu_set_history (GTK_OPTION_MENU (type_option), GFLOPPY_E2FS);
+
+	g_free (config.default_fs);
+
+	l = gtk_radio_button_get_group (GTK_RADIO_BUTTON (quick_format_rb));
+	default_mode_rb = (GtkWidget *) g_slist_nth (l, GFLOPPY_FORMAT_THOROUGH - config.default_formatting_mode)->data;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (default_mode_rb), TRUE);
+
+	/* check the UI consistency for a couple of settings */
+	g_signal_emit_by_name (type_option, "changed", NULL);
+
+	/* init_commands checks for at least one of the mke2fs or mkdosfs/mformat commands */
+	if (!floppy.mke2fs_cmd || !(floppy.mkdosfs_cmd || floppy.mformat_cmd)) {
+		GtkWidget *menu;
+		GtkWidget *menuitem;
+		const gchar *text;
+
+		/* Well, is there's any proper solution ? */
+		menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (type_option));
+		menuitem = GTK_WIDGET (g_list_nth (GTK_MENU_SHELL (menu)->children, floppy.mke2fs_cmd ? 0 : 1)->data);
+		text = gtk_label_get_text (GTK_LABEL (GTK_BIN (menuitem)->child));
+
+		fs_entry = glade_xml_get_widget (xml, "fs_entry");
+		gtk_entry_set_text (GTK_ENTRY (fs_entry), text);
+
+		gtk_option_menu_set_history (GTK_OPTION_MENU (type_option), floppy.mke2fs_cmd ? 1 : 0);
 
 		gtk_widget_hide (type_option);
-		gtk_widget_show (ext2_entry);
+		gtk_widget_show (fs_entry);
 	}
 
-	while ((button = gtk_dialog_run (GTK_DIALOG (toplevel))) >= 0) {
-		GtkWidget *density_option;
-		GtkWidget *dialog;
+	gtk_widget_show (GTK_WIDGET (toplevel));
 
-		if (button == GTK_RESPONSE_CLOSE) 
+	gtk_main ();
+
+	/* Sync config from UI before saving it */
+	switch (gtk_option_menu_get_history (GTK_OPTION_MENU (type_option))) {
+		case GFLOPPY_E2FS:
+			config.default_fs = GFLOPPY_CONFIG_FS_EXT2;
 			break;
 
-		if (button == GTK_RESPONSE_HELP) {
-			GError *error;
-
-			error = NULL;
-			gnome_help_display ("gfloppy", NULL, &error);
-			if (error) {
-				GtkWidget *dialog;
-
-				dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-								 0,
-								 GTK_MESSAGE_ERROR,
-								 GTK_BUTTONS_CLOSE,
-								 _("Could not display help for the "
-								   "floppy formatter.\n"
-								   "%s"),
-								 error->message);
-
-				g_signal_connect_swapped (dialog, "response",
-							  G_CALLBACK (gtk_widget_destroy),
-							  dialog);
-				gtk_widget_show (dialog);
-
-				g_error_free (error);
-			}
-
-			continue;
-		}
-		density_option = glade_xml_get_widget (xml, "density_option");
-		g_assert (density_option != NULL);
-		g_assert (quick_format_button != NULL);
-
-		gtk_widget_set_sensitive (toplevel, FALSE);
-
-		if (floppy.mformat_cmd) {
-			/* Check to see which one is selected. */
-			floppy.type = gtk_option_menu_get_history (GTK_OPTION_MENU (type_option));
-		} else {
-			floppy.type = GFLOPPY_E2FS;
-		}
-		floppy.size = gtk_option_menu_get_history (GTK_OPTION_MENU (density_option));
-		set_floppy_extended_device ();
-		floppy.quick_format = GTK_TOGGLE_BUTTON (quick_format_button)->active;
-
-		start_format ();
-		if (!toplevel)
+		case GFLOPPY_FAT:
+			config.default_fs = GFLOPPY_CONFIG_FS_FAT;
 			break;
-		gtk_widget_set_sensitive (toplevel, TRUE);
-		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_QUESTION,
-						 GTK_BUTTONS_YES_NO,
-						 _("Format another floppy?"));
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES)
-			exit (0);
-		gtk_widget_destroy (dialog);
 	}
+
+	config.default_formatting_mode = get_selected_formatting_mode ();
+
+	gfloppy_config_save (&config, client);
+
+	g_object_unref (G_OBJECT (client));
+
 	return 0;
 }
 
